/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-10-22 15:26:41.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.service.impl;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.core.print.common.Constants;
import cn.com.qmth.examcloud.core.print.common.jpa.OrderBuilder;
import cn.com.qmth.examcloud.core.print.common.jpa.SearchBuilder;
import cn.com.qmth.examcloud.core.print.common.jpa.SpecUtils;
import cn.com.qmth.examcloud.core.print.common.utils.Check;
import cn.com.qmth.examcloud.core.print.entity.ExamQuestionStructure;
import cn.com.qmth.examcloud.core.print.entity.ExamStructure;
import cn.com.qmth.examcloud.core.print.enums.ExamType;
import cn.com.qmth.examcloud.core.print.repository.ExamStructureRepository;
import cn.com.qmth.examcloud.core.print.service.ExamStructureService;
import cn.com.qmth.examcloud.core.print.service.bean.examstructure.ExamStructureConvert;
import cn.com.qmth.examcloud.core.print.service.bean.examstructure.ExamStructureInfo;
import cn.com.qmth.examcloud.core.print.service.bean.examstructure.ExamStructureQuery;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

/**
 * @author: fengdesheng
 * @since: 2018/10/17
 * @update: weiwenhai
 * @date: 2018.10.30
 */
@Service
public class ExamStructureServiceImpl implements ExamStructureService {
    private static final Logger log = LoggerFactory.getLogger(ExamStructureServiceImpl.class);
    @Autowired
    private ExamStructureRepository examStructureRepository;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Override
    public Page<ExamStructureInfo> getExamStructureList(ExamStructureQuery query) {
        Check.isNull(query, "查询参数不能为空！");
        Check.isEmpty(query.getOrgId(), "学校ID不能为空！");
        //查询条件
        SearchBuilder searches = new SearchBuilder();
        searches.eq("orgId", query.getOrgId());
        if (query.getExamId() != null) {
            searches.eq("examId", query.getExamId());
        }
        Specification<ExamStructure> spec = SpecUtils.buildSearchers(ExamStructure.class, searches.build());
        //排序条件
        OrderBuilder orders = new OrderBuilder().desc("id");
        //分页条件
        Pageable pageable = SpecUtils.buildPageable(query.getPageNo(), query.getPageSize(), orders.build());
        Page<ExamStructure> page = examStructureRepository.findAll(spec, pageable);
        return ExamStructureConvert.ofPage(page);
    }

    @Override
    public void saveExamStructure(ExamStructureInfo info) {
        Check.isNull(info.getOrgId(), "学校ID不能为空！");
        Check.isBlank(info.getOrgName(), "学校名称不能为空！");
        Check.isNull(info.getExamId(), "考试ID不能为空！");
        Check.isBlank(info.getExamName(), "考试名称不能为空！");

        ExamQuestionStructure questionStructure = info.getQuestionStructure();
        Check.isNull(questionStructure, "题数设置信息不能空！");
        Check.isNull(questionStructure.getSingleChoiceTotal(), "单选题数量不能为空！");
        Check.isNull(questionStructure.getMultipleChoiceTotal(), "多选题数量不能为空！");
        Check.isNull(questionStructure.getBoolQuestionTotal(), "判断题数量不能为空！");

        ExamStructure entity = ExamStructureConvert.of(info);
        entity.setExamType(ExamType.TRADITION.name());
        examStructureRepository.save(entity);
    }

    @Override
    public void deleteExamStructure(Long[] ids) {
        if (ids == null || ids.length == 0) {
            return;
        }
        for (Long id : ids) {
            try {
                examStructureRepository.delete(id);
            } catch (Exception e) {
                // ignore: record not exist
                log.warn(e.getMessage());
            }
        }
    }

    @Override
    public void copyExamStructure(ExamStructureInfo info) {
        //数据校验
        Check.isNull(info.getOrgId(), "原学校ID不能为空！");
        Check.isNull(info.getExamId(), "原考试ID不能为空！");
        Check.isNull(info.getNewOrgId(), "新学校ID不能为空！");
        Check.isBlank(info.getNewOrgName(), "新学校名称不能为空！");
        Check.isNull(info.getExamId(), "新考试ID不能为空！");
        Check.isBlank(info.getNewExamName(), "新考试名称不能为空！");

        ExamStructure newStructure = examStructureRepository.findByExamIdAndOrgId(info.getNewExamId(), info.getNewOrgId());
        if (newStructure != null) {
            throw new StatusException(Constants.PRT_CODE_500, "新学校考试下已有结构信息，不可复用！");
        }

        ExamStructure oldStructure = examStructureRepository.findByExamIdAndOrgId(info.getExamId(), info.getOrgId());
        if (oldStructure == null) {
            throw new StatusException(Constants.PRT_CODE_500, "原结构信息不存在！");
        }

        newStructure = new ExamStructure();
        newStructure.setId(null);
        newStructure.setOrgId(info.getNewOrgId());
        newStructure.setOrgName(info.getNewOrgName());
        newStructure.setExamId(info.getNewExamId());
        newStructure.setExamName(info.getNewExamName());
        newStructure.setExamType(oldStructure.getExamType());
        newStructure.setStruct(oldStructure.getStruct());
        examStructureRepository.save(newStructure);
    }

    @Override
    public void syncExamStructureExamName(Long examId, String examName) {
        String updateSql = String.format("UPDATE ec_prt_exam_structure SET exam_name = '%s' WHERE exam_id = %s", examName, examId);
        jdbcTemplate.update(updateSql);
    }

    @Override
    public ExamStructureInfo getExamStructure(ExamStructureQuery query) {
        Check.isNull(query, "查询参数不能为空！");
        Check.isEmpty(query.getOrgId(), "学校ID不能为空！");
        Check.isEmpty(query.getExamId(), "考试ID不能为空！");
        ExamStructure entity = examStructureRepository.findByExamIdAndOrgId(query.getExamId(), query.getOrgId());
        if (entity == null) {
            return null;
        }
        return ExamStructureConvert.of(entity);
    }

}