/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-10-17 16:31:14.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.service.impl;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.core.print.common.jpa.OrderBuilder;
import cn.com.qmth.examcloud.core.print.common.jpa.SearchBuilder;
import cn.com.qmth.examcloud.core.print.common.jpa.SpecUtils;
import cn.com.qmth.examcloud.core.print.common.utils.Check;
import cn.com.qmth.examcloud.core.print.entity.PrintingProject;
import cn.com.qmth.examcloud.core.print.enums.ExamType;
import cn.com.qmth.examcloud.core.print.repository.PrintingProjectRepository;
import cn.com.qmth.examcloud.core.print.service.CommonService;
import cn.com.qmth.examcloud.core.print.service.PrintingProjectService;
import cn.com.qmth.examcloud.core.print.service.bean.common.ExamInfo;
import cn.com.qmth.examcloud.core.print.service.bean.printingproject.PrintingProjectConvert;
import cn.com.qmth.examcloud.core.print.service.bean.printingproject.PrintingProjectInfo;
import cn.com.qmth.examcloud.core.print.service.bean.printingproject.PrintingProjectQuery;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.List;

import static cn.com.qmth.examcloud.core.print.common.Constants.PRT_CODE_500;

/**
 * 印刷项目相关接口
 *
 * @author: fengdesheng
 * @since: 2018/10/17
 */
@Service
public class PrintingProjectServiceImpl implements PrintingProjectService {
    private static final Logger log = LoggerFactory.getLogger(PrintingProjectServiceImpl.class);
    @Autowired
    private PrintingProjectRepository printingProjectRepository;
    @Autowired
    private CommonService commonService;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Override
    public Page<PrintingProjectInfo> getPrintingProjectList(PrintingProjectQuery query) {
        Check.isNull(query, "查询参数不能为空！");
        //查询条件
        SearchBuilder searches = new SearchBuilder();
        if (query.getOrgId() != null) {
            searches.eq("orgId", query.getOrgId());
        }
        if (query.getExamId() != null) {
            searches.eq("examId", query.getExamId());
        }
        if (query.getPmId() != null) {
            searches.eq("pmId", query.getPmId());
        }
        if (query.getSupplierId() != null) {
            searches.eq("supplierId", query.getSupplierId());
        }
        Specification<PrintingProject> spec = SpecUtils.buildSearchers(PrintingProject.class, searches.build());
        //排序条件
        OrderBuilder orders = new OrderBuilder().desc("id");
        //分页条件
        Pageable pageable = SpecUtils.buildPageable(query.getPageNo(), query.getPageSize(), orders.build());
        Page<PrintingProject> page = printingProjectRepository.findAll(spec, pageable);
        return PrintingProjectConvert.ofPage(page);
    }

    @Override
    public PrintingProjectInfo getPrintingProjectById(Long id) {
        Check.isNull(id, "印刷项目ID不能为空！");
        PrintingProject project = printingProjectRepository.findOne(id);
        if (project == null) {
            throw new StatusException(PRT_CODE_500, "印刷项目信息不存在！");
        }
        return PrintingProjectConvert.of(project);
    }

    @Override
    public PrintingProject getPrintingProjectByOrgIdAndExamId(Long orgId, Long examId) {
        SearchBuilder searches = new SearchBuilder()
                .eq("orgId", orgId)
                .eq("examId", examId);
        Specification<PrintingProject> spec = SpecUtils.buildSearchers(PrintingProject.class, searches.build());
        return printingProjectRepository.findOne(spec);
    }

    @Override
    public void updatePrintingProject(PrintingProjectInfo info) {
        Check.isNull(info.getId(), "印刷项目ID不能为空！");
        Check.isNull(info.getSupplierId(), "供应商ID不能为空！");
        Check.isBlank(info.getSupplierName(), "供应商名称不能为空！");
        Check.isNull(info.getPmId(), "项目经理ID不能为空！");
        Check.isBlank(info.getPmName(), "项目经理名称不能为空！");
        Check.isNull(info.getPrepareStartTime(), "准备开始时间不能为空！");
        Check.isNull(info.getPrepareEndTime(), "准备结束时间不能为空！");
        Check.isNull(info.getPrintStartTime(), "印刷开始时间不能为空！");
        Check.isNull(info.getPrintEndTime(), "印刷结束时间不能为空！");
        Check.isNull(info.getMailStartTime(), "邮寄开始时间不能为空！");
        Check.isNull(info.getMailEndTime(), "邮寄结束时间不能为空！");

        PrintingProject entity = printingProjectRepository.findOne(info.getId());
        if (entity == null) {
            throw new StatusException(PRT_CODE_500, "印刷项目信息不存在！");
        }

        entity.setSupplierId(info.getSupplierId());
        entity.setSupplierName(info.getSupplierName());
        entity.setPmId(info.getPmId());
        entity.setPmName(info.getPmName());
        entity.setPrepareStartTime(info.getPrepareStartTime());
        entity.setPrepareEndTime(info.getPrepareEndTime());
        entity.setPrintStartTime(info.getPrintStartTime());
        entity.setPrintEndTime(info.getPrintEndTime());
        entity.setMailStartTime(info.getMailStartTime());
        entity.setMailEndTime(info.getMailEndTime());
        printingProjectRepository.save(entity);
    }

    @Override
    public void syncPrintingProject(ExamInfo examInfo) {
        Check.isNull(examInfo.getOrgId(), "学校ID不能为空！");
        Check.isBlank(examInfo.getOrgName(), "学校名称不能为空！");
        Check.isNull(examInfo.getExamId(), "考试ID不能为空！");
        Check.isBlank(examInfo.getExamName(), "考试名称不能为空！");

        SearchBuilder searches = new SearchBuilder()
                .eq("orgId", examInfo.getOrgId())
                .eq("examId", examInfo.getExamId());
        Specification<PrintingProject> spec = SpecUtils.buildSearchers(PrintingProject.class, searches.build());

        //某学校某考试仅当作一个项目
        PrintingProject project = printingProjectRepository.findOne(spec);
        if (project != null) {
            //更新学校名称、考试名称等信息
            project.setOrgName(examInfo.getOrgName());
            project.setExamName(examInfo.getExamName());
            printingProjectRepository.save(project);
            return;
        }

        //新增印刷项目信息
        project = new PrintingProject();
        BeanUtils.copyProperties(examInfo, project);
        project.setCompleted(false);
        printingProjectRepository.save(project);
    }

    @Override
    public void syncAllPrintingProject() {
        //获取所有"传统"考试列表
        log.debug("syncAllPrintingProject...");
        List<ExamInfo> list = commonService.getExamList(ExamType.TRADITION.name());
        if (list != null && !list.isEmpty()) {
            for (ExamInfo info : list) {
                syncPrintingProject(info);
            }
        }
    }

    @Override
    public void syncPrintingProjectOrgName(Long orgId, String orgName) {
        if (orgId == null || StringUtils.isBlank(orgName)) {
            return;
        }
        String updateSql = String.format("UPDATE ec_prt_project SET org_name = '%s' WHERE org_id = %s", orgName, orgId);
        jdbcTemplate.update(updateSql);
    }

    @Override
    public void syncPrintingProjectExamName(Long examId, String examName) {
        if (examId == null || StringUtils.isBlank(examName)) {
            return;
        }
        String updateSql = String.format("UPDATE ec_prt_project SET exam_name = '%s' WHERE exam_id = %s", examName, examId);
        jdbcTemplate.update(updateSql);
    }

    @Override
    public void syncPrintingProjectPmName(Long pmId, String pmName) {
        if (pmId == null || StringUtils.isBlank(pmName)) {
            return;
        }
        String updateSql = String.format("UPDATE ec_prt_project SET pm_name = '%s' WHERE pm_id = %s", pmName, pmId);
        jdbcTemplate.update(updateSql);
    }

    @Override
    public void syncPrintingProjectSupplierName(Long supplierId, String supplierName) {
        if (supplierId == null || StringUtils.isBlank(supplierName)) {
            return;
        }
        String updateSql = String.format("UPDATE ec_prt_project SET supplier_name = '%s' WHERE supplier_id = %s", supplierName, supplierId);
        jdbcTemplate.update(updateSql);
    }

}